import { Component, Input, Output, EventEmitter } from '@angular/core';
import './jsoneditor.min.js';
declare var JSONEditor: any; //here or custom-typings.d.ts but current way webstorm wont give warning.

//require('./jsoneditor.min.js'); //Works too
// https://github.com/AngularClass/angular2-webpack-starter/issues/570

//var JSONEditor = require('json-editor');
// http://stackoverflow.com/questions/32173118/how-do-i-import-a-library-into-node-without-a-typescript-tsd-definition

//import * as JSONEditor from 'json-editor';
//declare var JSONEditor:any;

// Cheez-o-max way to import js lib that don't provide their own typings (directly imported in src/index.html for now)
// http://stackoverflow.com/questions/27417107/how-use-an-external-non-typescript-library-from-typescript-without-d-ts
// https://github.com/jdorn/json-editor/issues/590
// We could also write their typings by providing ambient definition
// Exeample at https://github.com/im-tollu/broken-import/blob/master/app/app.component.ts

@Component({
  selector: 'annotation-form',
  styles: [`
  `],
  template: `
    <h1>Annotation Form</h1>
    <div id="editor_holder"></div>
    <button (click)="saveAnnotation()" type="button" title="Save annotation & schema" class="btn btn-default"><i class="glyphicon glyphicon-save"></i> Save annotation & schema</button>
  `
})
export class AnnotationForm {
  @Output() onSave: EventEmitter<Object> = new EventEmitter<Object>();
  @Input('schema') schema: Object;
  @Input('annotation') annotation: Object;
  public editedAnnotation: Object = {};
  public editedSchema: Object = {};
  private editor: any = null;
  private mockAnnotation: Object = {};
  private mockJsonSchema: Object = {};

  constructor() {
    this.mockAnnotation = {
      "name": "Jeremy Dorn",
      "age": 25,
      "favorite_color": "#ffa500",
      "location": {
        "city": "San Francisco",
        "state": "CA",
        "citystate": "San Francisco, CA"
      },
      "pets": [
        {
          "type": "dog",
          "name": "Walter"
        }
      ]
    };

    this.mockJsonSchema = {
      "title": "Person",
      "type": "object",
      "properties": {
        "name": {
          "type": "string",
          "description": "First and Last name",
          "minLength": 4,
        },
        "age": {
          "type": "integer",
          "minimum": 18,
          "maximum": 99
        },
        "favorite_color": {
          "type": "string",
          "format": "color",
          "title": "favorite color",
        },
        "gender": {
          "type": "string",
          "enum": [
            "male",
            "female"
          ]
        },
        "location": {
          "type": "object",
          "title": "Location",
          "properties": {
            "city": {
              "default": "San Francisco",
              "type": "string",
            },
            "state": {
              "type": "string",
            },
            "citystate": {
              "type": "string",
              "description": "This is generated automatically from the previous two fields",
              "template": "{{city}}, {{state}}",
              "watch": {
                "city": "location.city",
                "state": "location.state"
              }
            }
          }
        },
        "pets": {
          "type": "array",
          "format": "table",
          "title": "Pets",
          "uniqueItems": true,
          "items": {
            "type": "object",
            "title": "Pet",
            "properties": {
              "type": {
                "type": "string",
                "enum": [
                  "cat",
                  "dog",
                  "bird",
                  "reptile",
                  "other"
                ]
                /*"default": "dog"*/
              },
              "name": {
                "type": "string"
              }
            }
          }
          /*"default": [
           {
           "type": "dog",
           "name": "Walter"
           }
           ]*/
        }
      }
    };
  }

  injectAnnotationInSchema(schema: Object, annotation: Object){
    //TODO: Inject annotation in default fields
    return schema;
  }

  saveAnnotation(): void{
    if(this.editor.validate()){
      this.editedAnnotation = this.editor.getValue();
      this.editedSchema = this.editor.schema;
      //TODO: Schema probably must be cleaned before being returned(propertyOrder, default values, etc.)
      this.onSave.emit({ annotation: this.editedAnnotation, schema: this.editedSchema });
    }
  }

  ngOnInit() {
    //I wish their was a better way of doing this: http://stackoverflow.com/questions/35528395/angular-2-make-input-on-directive-required
    //Validate required Inputs and Outputs are defined
    if (!this.onSave) throw new Error("Event 'onSave' is required");
    if (!this.schema) throw new Error("Attribute 'schema' is required");
    if (!this.annotation) throw new Error("Attribute 'annotation' is required");

    var element = document.getElementById('editor_holder');

    const options = {
      ajax: false, //	If true, JSON Editor will load external URLs in $ref via ajax.	false
      disable_array_add: false, //	If true, remove all "add row" buttons from arrays.	false
      disable_array_delete: false, //	If true, remove all "delete row" buttons from arrays.	false
      disable_array_reorder: false, //	If true, remove all "move up" and "move down" buttons from arrays.	false
      disable_collapse: false, //	If true, remove all collapse buttons from objects and arrays.	false
      disable_edit_json: true, //	If true, remove all Edit JSON buttons from objects.	false
      disable_properties: false, //	If true, remove all Edit Properties buttons from objects.	false
      form_name_root: 'form_name_root', //	The first part of the `name` attribute of form inputs in the editor. An full example name is `root[person][name]` where "root" is the form_name_root.	root
      iconlib: 'bootstrap3', //	The icon library to use for the editor. See the CSS Integration section below for more info.	null
      no_additional_properties: false, //	If true, objects can only contain properties defined with the properties keyword.	false
      refs: {}, //	An object containing schema definitions for URLs. Allows you to pre-define external schemas.	{}
      required_by_default: false, //	If true, all schemas that don't explicitly set the required property will be required.	false
      keep_oneof_values: true, //	If true, makes oneOf copy properties over when switching.	true
      schema: this.injectAnnotationInSchema(this.mockJsonSchema, this.mockAnnotation), //	A valid JSON Schema to use for the editor. Version 3 and Version 4 of the draft specification are supported.	{}
      show_errors: "interaction", //	When to show validation errors in the UI. Valid values are interaction, change, always, and never.	"interaction", "change", "always" and "never".
      startval: null, //	Seed the editor with an initial value. This should be valid against the editor's schema.	null
      template: 'default', //	The JS template engine to use. See the Templates and Variables section below for more info.	default
      theme: 'bootstrap3' //	The CSS theme to use. See the CSS Integration section below for more info.	html
    };
    this.editor = new JSONEditor(element, options);
  }
  ngOnDestroy() {
    // Speak now or forever hold your peace
  }
  ngDoCheck() {
    // Custom change detection
  }
  ngOnChanges(changes) {
    // Called right after our bindings have been checked but only
    // if one of our bindings has changed.
    //
    // changes is an object of the format:
    // {
    //   'prop': PropertyUpdate
    // }
  }
  ngAfterContentInit() {
    // Component content has been initialized
  }
  ngAfterContentChecked() {
    // Component content has been Checked
  }
  ngAfterViewInit() {
    // Component views are initialized
  }
  ngAfterViewChecked() {
    var yolo;
    // Component views have been checked
  }

}
